//+------------------------------------------------------------------+
//|                                               MovingAverages.mqh |
//|                   Copyright 2009-2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "2009, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"

#include <include BASE.mqh> 

/*
#define MODE_SMA  0
#define MODE_EMA  1
#define MODE_SMMA 2
#define MODE_LWMA 3
*/
#define MODE_DEMA 4
#define MODE_TEMA 5
#define MODE_LRMA 6
#define MODE_HMA 7
#define MODE_FRAMA 8
#define MODE_VIDYA 9
#define MODE_AMA 10
#define MODE_JMA 11

   //-------------------------
enum mis_MA_METODOS {SMA, EMA, SMMA, LWMA, DEMA, TEMA, LRMA, HMA, FRAMA, VIDYA, AMA, JMA};

//----------------------------- ANGULO DE MEDIA MOVIL ---------------------------------------------

double anguloMA(double &arrayValores[], int indVela, ENUM_TIMEFRAMES marco, int barraSalto= 2, double factor_visual= 0.80, string simb="")
{                     //DEVUELVE EL NGULO FORMADO POR LA LINEA DE UNA MEDIA MVIL, expresado en grados milesimales.
	double MAactual= 0, MAprevia= 0;  //medias calculadas para la barra 0 y la barra 2. La tangente es (incremento en puntos / minutos transcurridos)
	double incremento= 0, tangente=0, angulo=0;
	if(simb=="") simb= Symbol();
   MAactual= arrayValores[indVela];
	MAprevia= arrayValores[indVela+barraSalto];
	incremento= (MAactual - MAprevia) / valorPunto(simb);    //Point()
	tangente= incremento / (barraSalto*PeriodSeconds(marco)/60);
	angulo= MathArctan(tangente) / M_PI * 180 * factor_visual;         //ngulo en grados sexagesimales; si multiplicamos por 3200, milesimales.
	return(angulo);
}

//----------------------------------------  SELECCIN TIPO MEDIA MVIL  ------------------------------------------
int punteroAtipoMA(string simb, ENUM_TIMEFRAMES marco, int periodo, int metodo, ENUM_APPLIED_PRICE precioMA)
{
   int punt= -1, despl= 0;
   ENUM_MA_METHOD metEnum= defMetodoEnum(metodo);
   if (simb=="") simb= _Symbol;
   if (metodo<=3 && metodo>=0) punt=iMA(simb,marco,periodo,despl,metEnum,precioMA);
   else if (metodo==4) punt= iCustom(simb,marco,"DEMA",periodo,despl,precioMA);
   else if (metodo==5) punt= iCustom(simb,marco,"TEMA",periodo,despl,precioMA);
   else if (metodo==6) punt= iCustom(simb,marco,"LRMA",periodo,despl,precioMA);
   else if (metodo==7) punt= iCustom(simb,marco,"HMA",periodo,despl,precioMA);
   return(punt);

}

//+------------------------------------- DEFINE ETIQUETA de MA --------------------------------------+
string etiquetaMA(mis_MA_METODOS metodo, int MAPeriodo, ENUM_APPLIED_PRICE MAPrecio, int AMArapida= 5, int AMAlenta= 14)
{
   string etiqueta;
   //int AMArapida   = 2;
   //int AMAlenta    = 30;
   int CMOperiodo  = 9;
   switch(metodo)
     {
      case SMA:
         etiqueta="SMA("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case EMA:
         etiqueta="EMA("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case SMMA:
         etiqueta="SMMA("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case LWMA:
         etiqueta="LWMA("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case AMA:
         etiqueta="AMA("+IaS(MAPeriodo)+","+IaS(AMArapida)+","+IaS(AMAlenta)+","+abrevPrecio(MAPrecio)+")";
         break;
      case DEMA:
         etiqueta="DEMA("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case FRAMA:
         etiqueta="FRAM("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case TEMA:
         etiqueta="TEMA("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case VIDYA:
         etiqueta="VIDYA("+IaS(MAPeriodo)+","+IaS(CMOperiodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case HMA:
         etiqueta="HMA("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case LRMA:
         etiqueta="LRMA("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
      case JMA:
         etiqueta="JMA("+IaS(MAPeriodo)+","+abrevPrecio(MAPrecio)+")";
         break;
     }
   return(etiqueta);
}

//| abrevPrecio                                                  |
//+------------------------------------------------------------------+
string abrevPrecio(ENUM_APPLIED_PRICE aPrice)
  {
   switch(aPrice)
     {
      case PRICE_CLOSE:
         return("C");
      case PRICE_HIGH:
         return("H");
      case PRICE_LOW:
         return("L");
      case PRICE_MEDIAN:
         return("M");
      case PRICE_OPEN:
         return("O");
      case PRICE_TYPICAL:
         return("T");
      case PRICE_WEIGHTED:
         return("W");
     }
   return("?");
  }

//+------------------------------------------------------------------+
//| Simple Moving Average                                            |
//+------------------------------------------------------------------+
double SimpleMA(const int position,const int period,const double &price[])
  {
//---
   double result=0.0;
//--- check position
   if(position>=period-1 && period>0)
     {
      //--- calculate value
      for(int i=0;i<period;i++) result+=price[position-i];
      result/=period;
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Exponential Moving Average                                       |
//+------------------------------------------------------------------+
double ExponentialMA(const int position,const int period,const double prev_value,const double &price[])
  {
//---
   double result=0.0;
//--- calculate value
   if(period>0)
     {
      double pr=2.0/(period+1.0);
      result=price[position]*pr+prev_value*(1-pr);
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Smoothed Moving Average                                          |
//+------------------------------------------------------------------+
double SmoothedMA(const int position,const int period,const double prev_value,const double &price[])
  {
//---
   double result=0.0;
//--- check position
   if(period>0)
     {
      if(position==period-1)
        {
         for(int i=0;i<period;i++) result+=price[position-i];
         result/=period;
        }
      if(position>=period)
         result=(prev_value*(period-1)+price[position])/period;
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Linear Weighted Moving Average                                   |
//+------------------------------------------------------------------+
double LinearWeightedMA(const int position,const int period,const double &price[])
  {
//---
   double result=0.0,sum=0.0;
   int    i,wsum=0;
//--- calculate value
   if(position>=period-1 && period>0)
     {
      for(i=period;i>0;i--)
        {
         wsum+=i;
         sum+=price[position-i+1]*(period-i+1);
        }
      result=sum/wsum;
     }
//---
   return(result);
  }
//+------------------------------------------------------------------+
//| Simple moving average on price array                             |
//+------------------------------------------------------------------+
int SimpleMAOnBuffer(const int rates_total,const int prev_calculated,const int begin,
                     const int period,const double& price[],double& buffer[])
  {
   int i,limit;
//--- check for data
   if(period<=1 || rates_total-begin<period) return(0);
//--- save as_series flags
   bool as_series_price=ArrayGetAsSeries(price);
   bool as_series_buffer=ArrayGetAsSeries(buffer);
   if(as_series_price)  ArraySetAsSeries(price,false);
   if(as_series_buffer) ArraySetAsSeries(buffer,false);
//--- first calculation or number of bars was changed
   if(prev_calculated==0) // first calculation
     {
      limit=period+begin;
      //--- set empty value for first bars
      for(i=0;i<limit-1;i++) buffer[i]=0.0;
      //--- calculate first visible value
      double firstValue=0;
      for(i=begin;i<limit;i++)
         firstValue+=price[i];
      firstValue/=period;
      buffer[limit-1]=firstValue;
     }
   else limit=prev_calculated-1;
//--- main loop
   for(i=limit;i<rates_total;i++)
      buffer[i]=buffer[i-1]+(price[i]-price[i-period])/period;
//--- restore as_series flags
   if(as_series_price)  ArraySetAsSeries(price,true);
   if(as_series_buffer) ArraySetAsSeries(buffer,true);
//---
    return(rates_total);
  }
//+------------------------------------------------------------------+
//|  Exponential moving average on price array                       |
//+------------------------------------------------------------------+
int ExponentialMAOnBuffer(const int rates_total,const int prev_calculated,const int begin,
                          const int period,const double& price[],double& buffer[])
  {
   int    i,limit;
//--- check for data
   if(period<=1 || rates_total-begin<period) return(0);
   double dSmoothFactor=2.0/(1.0+period);
//--- save as_series flags
   bool as_series_price=ArrayGetAsSeries(price);
   bool as_series_buffer=ArrayGetAsSeries(buffer);
   if(as_series_price)  ArraySetAsSeries(price,false);
   if(as_series_buffer) ArraySetAsSeries(buffer,false);
//--- first calculation or number of bars was changed
   if(prev_calculated==0)
     {
      limit=period+begin;
      //--- set empty value for first bars
      for(i=0;i<begin;i++) buffer[i]=0.0;
      //--- calculate first visible value
      buffer[begin]=price[begin];
      for(i=begin+1;i<limit;i++)
         buffer[i]=price[i]*dSmoothFactor+buffer[i-1]*(1.0-dSmoothFactor);
     }
   else limit=prev_calculated-1;
//--- main loop
   for(i=limit;i<rates_total;i++)
      buffer[i]=price[i]*dSmoothFactor+buffer[i-1]*(1.0-dSmoothFactor);
//--- restore as_series flags
   if(as_series_price)  ArraySetAsSeries(price,true);
   if(as_series_buffer) ArraySetAsSeries(buffer,true);
//---
    return(rates_total);
  }
//+------------------------------------------------------------------+
//|  Linear weighted moving average on price array                   |
//+------------------------------------------------------------------+
int LinearWeightedMAOnBuffer(const int rates_total,const int prev_calculated,const int begin,
                             const int period,const double& price[],double& buffer[],int &weightsum)
  {
   int        i,limit;
   double     sum;
//--- check for data
   if(period<=1 || rates_total-begin<period) return(0);
//--- save as_series flags
   bool as_series_price=ArrayGetAsSeries(price);
   bool as_series_buffer=ArrayGetAsSeries(buffer);
   if(as_series_price)  ArraySetAsSeries(price,false);
   if(as_series_buffer) ArraySetAsSeries(buffer,false);
//--- first calculation or number of bars was changed
   if(prev_calculated==0)
     {
      weightsum=0;
      limit=period+begin;
      //--- set empty value for first bars
      for(i=0;i<limit;i++) buffer[i]=0.0;
      //--- calculate first visible value
      double firstValue=0;
      for(i=begin;i<limit;i++)
        {
         int k=i-begin+1;
         weightsum+=k;
         firstValue+=k*price[i];
        }
      firstValue/=(double)weightsum;
      buffer[limit-1]=firstValue;
     }
   else limit=prev_calculated-1;
//--- main loop
   for(i=limit;i<rates_total;i++)
     {
      sum=0;
      for(int j=0;j<period;j++) sum+=(period-j)*price[i-j];
      buffer[i]=sum/weightsum;
     }
//--- restore as_series flags
   if(as_series_price)  ArraySetAsSeries(price,true);
   if(as_series_buffer) ArraySetAsSeries(buffer,true);
//---
    return(rates_total);
  }
//+------------------------------------------------------------------+
//|  Smoothed moving average on price array                          |
//+------------------------------------------------------------------+
int SmoothedMAOnBuffer(const int rates_total,const int prev_calculated,const int begin,
                       const int period,const double& price[],double& buffer[])
  {
   int i,limit;
//--- check for data
   if(period<=1 || rates_total-begin<period) return(0);
//--- save as_series flags
   bool as_series_price=ArrayGetAsSeries(price);
   bool as_series_buffer=ArrayGetAsSeries(buffer);
   if(as_series_price)  ArraySetAsSeries(price,false);
   if(as_series_buffer) ArraySetAsSeries(buffer,false);
//--- first calculation or number of bars was changed
   if(prev_calculated==0)
     {
      limit=period+begin;
      //--- set empty value for first bars
      for(i=0;i<limit-1;i++) buffer[i]=0.0;
      //--- calculate first visible value
      double firstValue=0;
      for(i=begin;i<limit;i++)
         firstValue+=price[i];
      firstValue/=period;
      buffer[limit-1]=firstValue;
     }
   else limit=prev_calculated-1;
//--- main loop
   for(i=limit;i<rates_total;i++)
      buffer[i]=(buffer[i-1]*(period-1)+price[i])/period;
//--- restore as_series flags
   if(as_series_price)  ArraySetAsSeries(price,true);
   if(as_series_buffer) ArraySetAsSeries(buffer,true);
//---
    return(rates_total);
  }


//+------------------------------------------------------------------+
//|  Calculation of moving average for MQL4 programs.                |
//+------------------------------------------------------------------+
double iMAMql4(string simb,ENUM_TIMEFRAMES marcoTmp,int iMAPeriod,int desplMA,ENUM_MA_METHOD ma_method,ENUM_APPLIED_PRICE precioAplicado,int nVela)
  {
   //ENUM_TIMEFRAMES TimFram=TFMigrate(TimFrame);
   double mass[1];
   int puntero=iMA(simb,marcoTmp,iMAPeriod,desplMA,ma_method,precioAplicado);
   if(puntero<0)
     {
      Print("Failed to create the iMA object: Error ",GetLastError());
      return(0);
     }
   else
       if (CopyBuffer(puntero, 0, nVela, 1, mass) <= 0)
    return(0); 
   return(mass[0]);
  }
//+------------------------------------------------------------------------+
//| Calculation of moving average for MQL4 programs on an array.           |
//+------------------------------------------------------------------------+
double iMAenArray(double &Array[],int total,int iMAPeriod,int desplMA,ENUM_MA_METHOD ma_method,int nVela)
  {
   double buf[];
   if(total>0 && total<=iMAPeriod) return(0);
   if(total==0) total=ArraySize(Array);
   if(ArrayResize(buf,total)<0) return(0);
   switch(ma_method)
     {
      case MODE_SMA :
        {
         double sum=0;
         int    i, pos=total-1;
         for(i=1;i<iMAPeriod;i++,pos--)
            sum+=Array[pos];
         while(pos>=0)
           {
            sum+=Array[pos];
            buf[pos]=sum/iMAPeriod;
            sum-=Array[pos+iMAPeriod-1];
            pos--;
           }
         return(buf[nVela+desplMA]);
        }
      case MODE_EMA :
        {
         double pr=2.0/(iMAPeriod+1);
         int    pos=total-2;
         while(pos>=0)
           {
            if(pos==total-2) buf[pos+1]=Array[pos+1];
            buf[pos]=Array[pos]*pr+buf[pos+1]*(1-pr);
            pos--;
           }
         return(buf[nVela+desplMA]);
        }
      case MODE_SMMA :
        {
         double sum=0;
         int    i,k,pos;
         pos=total-iMAPeriod;
         while(pos>=0)
           {
            if(pos==total-iMAPeriod)
              {
               for(i=0,k=pos;i<iMAPeriod;i++,k++)
                 {
                  sum+=Array[k];
                  buf[k]=0;
                 }
              }
            else sum=buf[pos+1]*(iMAPeriod-1)+Array[pos];
            buf[pos]=sum/iMAPeriod;
            pos--;
           }
         return(buf[nVela+desplMA]);
        }
      case MODE_LWMA :
        {
         double sum=0.0,lsum=0.0;
         double price;
         int    i,weight=0,pos=total-1;
         for(i=1;i<=iMAPeriod;i++,pos--)
           {
            price=Array[pos];
            sum+=price*i;
            lsum+=price;
            weight+=i;
           }
         pos++;
         i=pos+iMAPeriod;
         while(pos>=0)
           {
            buf[pos]=sum/weight;
            if(pos==0) break;
            pos--;
            i--;
            price=Array[pos];
            sum=sum-lsum+price*iMAPeriod;
            lsum-=Array[i];
            lsum+=price;
           }
         return(buf[nVela+desplMA]);
        }
      default: return(0);
     }
   return(0);
  }
//+------------------------------------------------------------------+
//| Exponential Average                                              |
//+------------------------------------------------------------------+
void EMA(double &Array[],int InpMAPeriod)
  {
   int count=ArraySize(Array)-1;
   double pr=2.0/(InpMAPeriod+1);
   count--;
   while(count>=0)
     {
      Array[count]=Array[count]*pr+Array[count+1]*(1-pr);
      count--;
     }
  }
//+------------------------------------------------------------------+
//|  Simple Average                                                  |
//+------------------------------------------------------------------+
void SimpleMA(double &price[],double &ExtLineBuffer[],int InpMAPeriod)
  {
   int rates_total=ArraySize(price);
   int i,ii;
   double Value;

   for(ii=0;ii<(rates_total-InpMAPeriod);ii++)
     {
      Value=0;
      for(i=0;i<InpMAPeriod;i++) Value+=price[ii+i];

      Value/=InpMAPeriod;

      ExtLineBuffer[ii]=Value;
     }
  }
//+------------------------------------------------------------------+
//| Smoothed Average                                                 |
//+------------------------------------------------------------------+
void SmoothedMA(double &price[],double &ExtLineBuffer[],int InpMAPeriod)
  {
   int rates_total=ArraySize(price)-1;
   int i,limit;
   limit=rates_total-InpMAPeriod;
   double firstValue=0;

   for(i=rates_total;i>limit;i--) firstValue+=price[i];
   firstValue/=InpMAPeriod;

   ExtLineBuffer[limit+1]=firstValue;

   for(i=limit;i>=0;i--) ExtLineBuffer[i]=(ExtLineBuffer[i+1]*(InpMAPeriod-1)+price[i])/InpMAPeriod;
//---
  }
//+------------------------------------------------------------------+
//| Linear Weighted Average                                          |
//+------------------------------------------------------------------+
void LWMA(double &price[],double &ExtLineBuffer[],int InpMAPeriod)
  {
   int rates_total=ArraySize(price)-1;
   int  limit=rates_total-InpMAPeriod;
   int i,weightsum;
   weightsum=0;
   double     sum;

   double firstValue=0;

   for(i=limit;i<=rates_total;i++)
     {
      int k=i-limit;
      weightsum+=k;
      firstValue+=k*price[i];
     }
   firstValue/=weightsum;
   ExtLineBuffer[limit+1]=firstValue;

//--- main loop
   for(i=limit;i>=0;i--)
     {
      sum=0;
      for(int j=0;j<InpMAPeriod;j++) sum+=(InpMAPeriod-j)*price[i+j];
      ExtLineBuffer[i]=sum/weightsum;
     }
  }
